@@ -210,6 +210,8 @@ def test_apply_basic_transform(path, transform, expected_result):
         (SVGPath(d="M1,2"), False),
         (SVGPath(d="M1,2 M3,4"), False),
         (SVGPath(d="M1,2 L3,4 Z"), True),
+        (SVGPath(d="M1,1 L2,1 L2,2 L1,2 Z", display="none"), False),
+        (SVGPath(style="display:none;fill:#F5FAFC;", d="M1,1 L2,1 L2,2 L1,2 Z"), False),
     ],
 )
 def test_might_paint(path, expected_result):
@@ -241,3 +243,81 @@ def test_apply_style_attribute(shape, expected):
     actual = shape.apply_style_attribute()
     assert actual == expected
     assert shape.apply_style_attribute(inplace=True) == expected
+
+
+@pytest.mark.parametrize(
+    "path, multiple_of, expected_result",
+    [
+        ("m1,1 2,0 1,3", 0.1, "m1,1 2,0 1,3"),
+        # why a multiple that divides evenly into 1 is a good idea
+        ("m1,1 2,0 1,3", 0.128, "m1.024,1.024 2.048,0 1.024,2.944"),
+    ],
+)
+def test_round_multiple(path: str, multiple_of: float, expected_result: str):
+    actual = SVGPath(d=path).round_multiple(multiple_of, inplace=True).d
+    print(f"A: {actual}")
+    print(f"E: {expected_result}")
+    assert actual == expected_result
+
+
+@pytest.mark.parametrize(
+    "shape, expected",
+    [
+        # neither fill nor stroke, unchanged
+        (
+            SVGPath(d="m1,1 2,0 1,3 z", fill="none", fill_opacity=0.0),
+            SVGPath(d="m1,1 2,0 1,3 z", fill="none", fill_opacity=0.0),
+        ),
+        # both fill and stroke, also unchanged
+        (
+            SVGPath(
+                d="m1,1 2,0 1,3 z",
+                fill="red",
+                fill_opacity=0.5,
+                stroke="black",
+                stroke_opacity=0.8,
+                opacity=0.9,
+            ),
+            SVGPath(
+                d="m1,1 2,0 1,3 z",
+                fill="red",
+                fill_opacity=0.5,
+                stroke="black",
+                stroke_opacity=0.8,
+                opacity=0.9,
+            ),
+        ),
+        # no stroke, only fill: merge and drop fill_opacity
+        (
+            SVGPath(
+                d="m1,1 2,0 1,3 z",
+                fill="red",
+                fill_opacity=0.5,
+                opacity=0.9,
+            ),
+            SVGPath(
+                d="m1,1 2,0 1,3 z",
+                fill="red",
+                opacity=0.45,  # 0.9 * 0.5
+            ),
+        ),
+        # no fill, only stroke: merge and drop stroke_opacity
+        (
+            SVGPath(
+                d="m1,1 2,0 1,3 z",
+                fill="none",
+                stroke="black",
+                stroke_opacity=0.3,
+                opacity=0.9,
+            ),
+            SVGPath(
+                d="m1,1 2,0 1,3 z",
+                fill="none",
+                stroke="black",
+                opacity=0.27,  # 0.9 * 0.3
+            ),
+        ),
+    ],
+)
+def test_normalize_opacity(shape, expected):
+    assert shape.normalize_opacity() == expected
